package = "oss"
version = "0.1.3"
url = "http://oss.aliyun.com"
license = "GPL version 2"
short_description = "Command line tool for managing Aliyun Open Storage Service."
long_description = """
osscmd lets you create/delete/list bucket and upload/download/copy/delete file from/to
        Aliyun OSS (Open Storage Service).
"""

